function [S2, PHI] = diffuse_sim(S, step, mode, nb, k)
% S         : pre-diffused similarity matrix, NxN
% step      : diffustion steps, 1x1
% mode      : mode for diffusion, 1x1
%             1 - S->S*(P^t)->S2
%             2 - S->P->P^t->D->S2
% nb        : number of neighbors for variance estimation, 1x1
% k         : constant before variance, 1x1
% S2        : post-diffused similarity matrix, NxN
%
% Author    : Jiayan Jiang
% Note      : The PSD property of S is not checked or enforced.

if (nargin <= 2)
    mode = 1;
end;

N = size(S, 1);

% stochastic matrix
if issparse(S)
    P = bsxfun(@times, S, 1./sum(S, 2));
else
    P = S .* repmat(1./sum(S, 2), 1, N);
end;

switch mode
    case 1
        S2 = S * (P ^ step);
        S2 = S2 ./ repmat(diag(S2), 1, N);
    case 2
        u = sum(S, 2)' / sum(S(:));
        P_ = P ^ step;
        PHI = P_' ./ repmat(sqrt(u'), 1, N);
        XX = sum(PHI.^2, 1);
        K = PHI'*PHI;
        D = repmat(XX, N, 1) + repmat(XX', 1, N) - 2*K;
        clear S P u P_ XX K;
        S2 = dist2sim(D, nb, k);
end;
