function S = dist2sim(D, nb, k)
% D         : pairwise distance matrix, NxN
% nb        : number of neighbors for variance estimation, 1x1
% k         : constant before variance, 1x1
% S         : pairwise similarity matrix, NxN
%
% Author    : Jiayan Jiang
% Note      : The PSD property of D is not checked or enforced.

N = size(D, 1);

D = (D+D')/2;       % enforce symmetricity
D(1:N+1:end) = 0;   % enforce reflexivity

[dumb, IDX] = sort(D, 2);

sig2 = 0;
for i = 1:N
    idx = IDX(i, :);
    idx(idx==i) = []; % self-exclusion
    sig2 = sig2 + sum(D(i, idx(1:nb)));
end;
sig2 = sig2 / (nb*N);

S = exp(-D ./ (k*sig2));
